import math
import os
import sys
import folium
import geopandas as gpd
import jenkspy
import numpy as np
from PyQt5.QtCore import QUrl, Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QPixmap, QColor, QIcon
from PyQt5.QtWebEngineWidgets import QWebEngineView
from PyQt5.QtWidgets import (
    QApplication, QMainWindow, QFileDialog, QPushButton, QLabel, QVBoxLayout,
    QLineEdit, QWidget, QComboBox, QMessageBox, QGroupBox, QTabWidget,
    QSpinBox, QHBoxLayout, QTextEdit, QAction, QScrollArea, QSizePolicy
)

# 默认单变量色带
DEFAULT_RAMP_COLORS = {
    "Viridis": ["#440154", "#472D7B", "#395F86", "#278084", "#1F9E89", "#35B779", "#6CCE59", "#B6DE2B", "#FDE725"],
    "Plasma": ["#0D0887", "#5B02A3", "#9A179B", "#CD376A", "#D1605D", "#D98E4C", "#E2B357", "#F3E185", "#FDE725"],
    "Inferno": ["#000004", "#3B0F70", "#8C2981", "#DA4167", "#F1605D", "#FB8761", "#FBB06B", "#FAD96D", "#FEE588"],
    "Magma": ["#000004", "#1C1044", "#5B0C59", "#990C6D", "#D02659", "#F17145", "#FEBE2A", "#FDF705", "#FCF8FB"],
    "Cividis": ["#00204C", "#2C3E70", "#576D8C", "#7FA5A8", "#A5CFAD", "#C8F0AA", "#ECF8A1", "#FFF79E", "#FFF6B5"],
    "Ocean": ["#E8E6F2", "#B5D3E7", "#82C0DB", "#4FADD0"],
    "Rose": ["#E8E6F2", "#E5B4D9", "#E181BF", "#DE4FA6"]
}


def get_expanded_colors(base_colors, target_count):
    if target_count <= len(base_colors):
        return base_colors[:target_count]
    else:
        expanded = []
        old_size = len(base_colors)
        for i in range(target_count):
            orig_idx = min(math.floor(i * old_size / target_count), old_size - 1)
            expanded.append(base_colors[orig_idx])
        return expanded


def hex_to_rgb(hex_color):
    hex_color = hex_color.lstrip('#')
    return tuple(int(hex_color[i:i + 2], 16) for i in (0, 2, 4))


def rgb_to_hex(rgb):
    return '#' + ''.join(f'{c:02X}' for c in rgb)


def blend_colors(hex1, hex2):
    rgb1 = hex_to_rgb(hex1)
    rgb2 = hex_to_rgb(hex2)
    blended = tuple((c1 + c2) // 2 for c1, c2 in zip(rgb1, rgb2))
    return rgb_to_hex(blended)


def create_color_icon(color, size=20):
    pixmap = QPixmap(size, size)
    pixmap.fill(QColor(color))
    return pixmap


def expand_matrix(matrix, new_size):
    old_size = len(matrix)
    expanded = []
    for i in range(new_size):
        row = []
        for j in range(new_size):
            old_i = min(int(i * old_size / new_size), old_size - 1)
            old_j = min(int(j * old_size / new_size), old_size - 1)
            row.append(matrix[old_i][old_j])
        expanded.append(row)
    return expanded


def create_bivariate_legend_html(breaks1, breaks2, color_matrix, var1_name="Variable 1", var2_name="Variable 2",
                                 legend_title="Bivariate Legend"):
    """
    根据断点和颜色矩阵生成双变量图例的 HTML 字符串
    """
    n_classes = len(breaks1) - 1
    legend_html = f"""
    <div style="
        position: fixed;
        bottom: 50px; left: 50px;
        z-index:9999;
        background-color: rgba(255, 255, 255, 0.8);
        padding: 5px;
        border: 2px solid grey;
        font-size: 10px;
    ">
      <h4 style="margin:2px; padding:0; text-align:center;">{legend_title}</h4>
      <p style="margin:2px; padding:0; text-align:center;">
        {var1_name} vs. {var2_name}
      </p>
      <table style="border-collapse: collapse; margin:0 auto;">
        <tr>
          <td></td>
    """
    for j in range(n_classes):
        legend_html += f"<th style='border:1px solid #999; padding:2px;'>{breaks2[j]:.2f} - {breaks2[j + 1]:.2f}</th>"
    legend_html += "</tr>"
    for i in range(n_classes):
        legend_html += f"<tr><th style='border:1px solid #999; padding:2px;'>{breaks1[i]:.2f} - {breaks1[i + 1]:.2f}</th>"
        for j in range(n_classes):
            color = color_matrix[i][j]
            legend_html += f"<td style='background:{color}; width:20px; height:20px; border:1px solid #999;'></td>"
        legend_html += "</tr>"
    legend_html += """
      </table>
    </div>
    """
    return legend_html


class BivariateMapApp(QMainWindow):
    def __init__(self):
        super().__init__()

        # 默认语言为英文
        self.current_language = "en"
        self.translations = {
            'en': {
                'window_title': 'InterCartoMap',
                'tab_data_selection': 'Data Selection',
                'tab_settings': 'Settings',
                'tab_map_generation': 'Map Generation',
                'file_selection_group': 'File Selection',
                'no_file_selected': 'No file selected',
                'choose_geojson_file': 'Select GeoJSON File',
                'variable_selection_group': 'Variable Selection',
                'select_variable_1': 'Select Variable 1:',
                'select_variable_2': 'Select Variable 2:',
                'break_settings_group': 'Break Settings',
                'select_break_method': 'Select Break Method:',
                'manual_input': 'Manual Input',
                'natural_breaks': 'Natural Breaks',
                'calculate_natural_breaks': 'Calculate Natural Breaks',
                'select_number_of_classes': 'Select Number of Classes (2-10):',
                'variable1_breaks': 'Variable 1 Breaks (comma-separated):',
                'variable2_breaks': 'Variable 2 Breaks (comma-separated):',
                'color_matrix_settings_group': 'Color Matrix Settings',
                'variable1_color_ramp': 'Variable 1 Color Ramp:',
                'variable2_color_ramp': 'Variable 2 Color Ramp:',
                'enter_custom_colors_var1': 'Enter custom colors for Variable 1 (comma-separated)',
                'enter_custom_colors_var2': 'Enter custom colors for Variable 2 (comma-separated)',
                'preview_color_matrix': 'Preview Color Matrix',
                'color_matrix_input_label': 'Color Matrix Input (each row corresponds to a category of Variable 1):',
                'color_matrix_preview_label': 'Color Matrix Preview:',
                'tooltip_settings_group': 'Interactive Tooltip Settings',
                'tooltip_instructions': 'Select fields to display and customize labels (up to 3):',
                'field_label': 'Field {num}:',
                'label_label': 'Label:',
                'generate_map': 'Generate Map',
                'save_map': 'Save Map',
                'please_select_geojson': 'Please select a GeoJSON file!',
                'please_calculate_breaks': 'Please calculate breaks first!',
                'map_generated_info': 'Map has been generated and previewed!',
                'map_saved_info': 'Map has been saved to: {path}',
                'save_failed': 'Save failed: {error}',
                'error_title': 'Error',
                'warning_title': 'Warning',
                'info_title': 'Information',
                'file_read_error': 'Failed to read file',
                'insufficient_columns': 'Data must have at least two columns!',
                'break_calculation_failed': 'Break calculation failed: {error}',
                'breaks_calculated': 'Natural breaks calculated and filled!',
                'legend_title': 'Bivariate Legend'
            },
            'zh': {
                'window_title': '双变量交互式变形地图',
                'tab_data_selection': '数据选择',
                'tab_settings': '设置',
                'tab_map_generation': '地图生成',
                'file_selection_group': '文件选择',
                'no_file_selected': '未选择文件',
                'choose_geojson_file': '选择 GeoJSON 文件',
                'variable_selection_group': '变量选择',
                'select_variable_1': '选择变量 1：',
                'select_variable_2': '选择变量 2：',
                'break_settings_group': '断点设置',
                'select_break_method': '选择断点方式：',
                'manual_input': '手动输入',
                'natural_breaks': '自然间断点法',
                'calculate_natural_breaks': '计算自然断点',
                'select_number_of_classes': '选择类别数量（2-10）：',
                'variable1_breaks': '变量 1 断点（逗号分隔）：',
                'variable2_breaks': '变量 2 断点（逗号分隔）：',
                'color_matrix_settings_group': '配色矩阵设置',
                'variable1_color_ramp': '变量1 色带选择：',
                'variable2_color_ramp': '变量2 色带选择：',
                'enter_custom_colors_var1': '请输入变量1的自定义颜色（逗号分隔）',
                'enter_custom_colors_var2': '请输入变量2的自定义颜色（逗号分隔）',
                'preview_color_matrix': '预览配色矩阵',
                'color_matrix_input_label': '配色矩阵输入（每行对应变量 1 的类别）：',
                'color_matrix_preview_label': '配色矩阵预览：',
                'tooltip_settings_group': '交互式提示标签设置',
                'tooltip_instructions': '选择要展示的字段并自定义标签（最多3个）：',
                'field_label': '字段{num}：',
                'label_label': '标签：',
                'generate_map': '生成地图',
                'save_map': '保存地图',
                'please_select_geojson': '请先选择 GeoJSON 文件！',
                'please_calculate_breaks': '请先计算断点！',
                'map_generated_info': '地图已生成并预览在界面中！',
                'map_saved_info': '地图已保存到：{path}',
                'save_failed': '保存失败：{error}',
                'error_title': '错误',
                'warning_title': '警告',
                'info_title': '成功',
                'file_read_error': '无法读取文件',
                'insufficient_columns': '数据中至少需要两个列！',
                'break_calculation_failed': '断点计算失败：{error}',
                'breaks_calculated': '自然断点已计算并填充！',
                'legend_title': '双变量图例'
            }
        }

        self.setWindowTitle(self.translations[self.current_language]['window_title'])
        self.setFixedSize(800, 900)

        # 设置窗口图标（图标文件在固定目录）
        icon_path = os.path.join(r"D:\Jupyter\遗产灾害", "icon.ico")
        self.setWindowIcon(QIcon(icon_path))

        # 添加菜单栏中的语言选择
        menubar = self.menuBar()
        language_menu = menubar.addMenu("Language")
        english_action = QAction("English", self)
        english_action.triggered.connect(lambda: self.change_language("en"))
        language_menu.addAction(english_action)
        chinese_action = QAction("中文", self)
        chinese_action.triggered.connect(lambda: self.change_language("zh"))
        language_menu.addAction(chinese_action)

        self.file_path = None
        self.gdf = None
        self.map_html = "InterCartoMap.html"

        main_layout = QVBoxLayout()
        self.tabs = QTabWidget()

        # ----- Tab 1: 数据选择 / Data Selection -----
        self.tab1 = QWidget()
        tab1_layout = QVBoxLayout()

        # 文件选择组
        self.file_group = QGroupBox()
        file_layout = QVBoxLayout()
        self.file_label = QLabel()
        self.file_button = QPushButton()
        self.file_button.clicked.connect(self.load_file)
        file_layout.addWidget(self.file_label)
        file_layout.addWidget(self.file_button)
        self.file_group.setLayout(file_layout)
        self.file_group.setFixedHeight(100)

        # 变量选择组
        self.var_group = QGroupBox()
        var_layout = QVBoxLayout()
        self.var1_label = QLabel()
        self.var1_combo = QComboBox()
        self.var2_label = QLabel()
        self.var2_combo = QComboBox()
        var_layout.addWidget(self.var1_label)
        var_layout.addWidget(self.var1_combo)
        var_layout.addWidget(self.var2_label)
        var_layout.addWidget(self.var2_combo)
        self.var_group.setLayout(var_layout)
        self.var_group.setFixedHeight(150)

        tab1_layout.addWidget(self.file_group)
        tab1_layout.addWidget(self.var_group)
        self.tab1.setLayout(tab1_layout)
        self.tabs.addTab(self.tab1, "")

        main_layout.addWidget(self.tabs)
        container = QWidget()
        container.setLayout(main_layout)
        self.setCentralWidget(container)

        # ----- Tab 2: 设置 / Settings -----
        self.tab2 = QWidget()
        tab2_layout = QVBoxLayout()

        # 断点设置组
        self.break_group = QGroupBox()
        break_layout = QVBoxLayout()
        self.break_method_label = QLabel()
        self.break_method_combo = QComboBox()
        self.break_method_combo.addItems([self.translations[self.current_language]['manual_input'],
                                          self.translations[self.current_language]['natural_breaks']])
        self.break_method_combo.currentIndexChanged.connect(self.toggle_break_method)
        self.calculate_button = QPushButton()
        self.calculate_button.clicked.connect(self.calculate_breaks)
        self.calculate_button.setEnabled(False)

        hlayout_count = QHBoxLayout()
        self.count_label = QLabel()
        self.break_count_spin = QSpinBox()
        self.break_count_spin.setRange(2, 10)
        self.break_count_spin.setValue(4)
        self.break_count_spin.valueChanged.connect(self.update_color_matrix_size)
        hlayout_count.addWidget(self.count_label)
        hlayout_count.addWidget(self.break_count_spin)

        self.break_input1_label = QLabel()
        self.break_input_1 = QLineEdit()
        self.break_input2_label = QLabel()
        self.break_input_2 = QLineEdit()
        break_layout.addWidget(self.break_method_label)
        break_layout.addWidget(self.break_method_combo)
        break_layout.addLayout(hlayout_count)
        break_layout.addWidget(self.calculate_button)
        break_layout.addWidget(self.break_input1_label)
        break_layout.addWidget(self.break_input_1)
        break_layout.addWidget(self.break_input2_label)
        break_layout.addWidget(self.break_input_2)
        self.break_group.setLayout(break_layout)

        # 配色矩阵设置组
        self.color_group = QGroupBox()
        color_layout = QVBoxLayout()

        hlayout_ramp = QHBoxLayout()
        self.ramp1_label = QLabel()
        self.ramp1_combo = QComboBox()
        model1 = QStandardItemModel()
        for ramp_name, colors in DEFAULT_RAMP_COLORS.items():
            item = QStandardItem(ramp_name)
            item.setIcon(QIcon(create_color_icon(colors[0])))
            model1.appendRow(item)
        custom_item = QStandardItem("自定义")
        model1.appendRow(custom_item)
        self.ramp1_combo.setModel(model1)
        self.ramp1_combo.currentTextChanged.connect(lambda text: self.toggle_custom_input(text, 1))
        hlayout_ramp.addWidget(self.ramp1_label)
        hlayout_ramp.addWidget(self.ramp1_combo)

        self.custom_ramp1_input = QLineEdit()
        self.custom_ramp1_input.setVisible(False)
        hlayout_ramp.addWidget(self.custom_ramp1_input)

        self.ramp2_label = QLabel()
        self.ramp2_combo = QComboBox()
        model2 = QStandardItemModel()
        for ramp_name, colors in DEFAULT_RAMP_COLORS.items():
            item = QStandardItem(ramp_name)
            item.setIcon(QIcon(create_color_icon(colors[0])))
            model2.appendRow(item)
        custom_item2 = QStandardItem("自定义")
        model2.appendRow(custom_item2)
        self.ramp2_combo.setModel(model2)
        self.ramp2_combo.currentTextChanged.connect(lambda text: self.toggle_custom_input(text, 2))
        hlayout_ramp.addWidget(self.ramp2_label)
        hlayout_ramp.addWidget(self.ramp2_combo)

        self.custom_ramp2_input = QLineEdit()
        self.custom_ramp2_input.setVisible(False)
        hlayout_ramp.addWidget(self.custom_ramp2_input)

        color_layout.addLayout(hlayout_ramp)

        self.preview_button = QPushButton()
        self.preview_button.clicked.connect(self.generate_color_matrix)
        color_layout.addWidget(self.preview_button)

        self.color_matrix_input_label = QLabel()
        self.color_matrix_input = QTextEdit()
        color_layout.addWidget(self.color_matrix_input_label)
        color_layout.addWidget(self.color_matrix_input)
        self.color_matrix_preview_label = QLabel()
        color_layout.addWidget(self.color_matrix_preview_label)

        # 使用 QScrollArea 包裹颜色矩阵预览控件
        self.color_preview = QLabel()
        self.color_preview.setTextFormat(Qt.RichText)
        self.color_preview.setAlignment(Qt.AlignLeft | Qt.AlignTop)
        self.color_preview.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.preview_scroll = QScrollArea()
        self.preview_scroll.setWidgetResizable(True)
        self.preview_scroll.setWidget(self.color_preview)
        color_layout.addWidget(self.preview_scroll)

        self.color_group.setLayout(color_layout)

        # 交互式提示标签设置组
        self.tooltip_group = QGroupBox()
        tooltip_layout = QVBoxLayout()
        self.tooltip_instructions = QLabel()
        tooltip_layout.addWidget(self.tooltip_instructions)
        self.tooltip_selectors = []
        self.tooltip_field_labels = []
        self.tooltip_label_labels = []
        for i in range(3):
            hlayout = QHBoxLayout()
            field_label = QLabel()
            combo = QComboBox()
            line_edit = QLineEdit()
            line_edit.setPlaceholderText("")
            hlayout.addWidget(field_label)
            hlayout.addWidget(combo)
            label_label = QLabel()
            hlayout.addWidget(label_label)
            hlayout.addWidget(line_edit)
            tooltip_layout.addLayout(hlayout)
            self.tooltip_selectors.append((combo, line_edit))
            self.tooltip_field_labels.append(field_label)
            self.tooltip_label_labels.append(label_label)
        self.tooltip_group.setLayout(tooltip_layout)

        tab2_layout.addWidget(self.break_group)
        tab2_layout.addWidget(self.color_group)
        tab2_layout.addWidget(self.tooltip_group)
        self.tab2.setLayout(tab2_layout)
        self.tabs.addTab(self.tab2, "")

        # ----- Tab 3: 地图生成 / Map Generation -----
        self.tab3 = QWidget()
        tab3_layout = QVBoxLayout()
        self.generate_button = QPushButton()
        self.generate_button.clicked.connect(self.generate_map)
        self.save_button = QPushButton()
        self.save_button.clicked.connect(self.save_map)
        self.save_button.setEnabled(False)
        self.map_preview = QWebEngineView()
        tab3_layout.addWidget(self.generate_button)
        tab3_layout.addWidget(self.map_preview)
        tab3_layout.addWidget(self.save_button)
        self.tab3.setLayout(tab3_layout)
        self.tabs.addTab(self.tab3, "")

        self.update_color_preview()
        self.update_ui_text()

        # 设置全局 QSS 样式
        self.setStyleSheet(self.get_stylesheet())

    def get_stylesheet(self):
        # QSS 样式，美化整体界面
        return """
        QMainWindow {
            background-color: #f0f0f0;
        }
        QGroupBox {
            font-weight: bold;
            border: 1px solid #d3d3d3;
            border-radius: 5px;
            margin-top: 10px;
            background-color: #ffffff;
        }
        QGroupBox::title {
            subcontrol-origin: margin;
            left: 10px;
            padding: 0 5px;
        }
        QLabel {
            font-size: 14px;
        }
        QLineEdit, QComboBox, QTextEdit, QSpinBox {
            font-size: 14px;
            padding: 5px;
            border: 1px solid #c0c0c0;
            border-radius: 3px;
            background-color: #ffffff;
        }
        QPushButton {
            background-color: #D6D6EF;
            color: #332C2B;
            border: none;
            padding: 8px 16px;
            font-size: 14px;
            border-radius: 5px;
        }
        QPushButton:hover {
            background-color: #D6D6EF;
        }
        QTabWidget::pane {
            border: 1px solid #dcdcdc;
            background-color: #ffffff;
        }
        QTabBar::tab {
            background: #e0e0e0;
            border: 1px solid #dcdcdc;
            padding: 10px;
            border-top-left-radius: 5px;
            border-top-right-radius: 5px;
            min-width: 100px;
        }
        QTabBar::tab:selected, QTabBar::tab:hover {
            background: #ffffff;
        }
        QMenuBar {
            background-color: #D6D6EF;
            color: #332C2B;
        }
        QMenuBar::item {
            background-color: #D6D6EF;
            padding: 4px 10px;
        }
        QMenuBar::item:selected {
            background-color: #D6D6EF;
        }
        QMenu {
            background-color: #ffffff;
            border: 1px solid #dcdcdc;
        }
        QMenu::item {
            padding: 4px 20px;
        }
        QMenu::item:selected {
            background-color: #e0e0e0;
        }
        """

    def change_language(self, lang):
        self.current_language = lang
        self.update_ui_text()

    def update_ui_text(self):
        tr = self.translations[self.current_language]
        self.setWindowTitle(tr['window_title'])
        # 更新各个选项卡的标题
        self.tabs.setTabText(0, tr['tab_data_selection'])
        self.tabs.setTabText(1, tr['tab_settings'])
        self.tabs.setTabText(2, tr['tab_map_generation'])
        # Tab 1
        self.file_group.setTitle(tr['file_selection_group'])
        self.file_label.setText(tr['no_file_selected'])
        self.file_button.setText(tr['choose_geojson_file'])
        self.var_group.setTitle(tr['variable_selection_group'])
        self.var1_label.setText(tr['select_variable_1'])
        self.var2_label.setText(tr['select_variable_2'])
        # Tab 2 - 断点设置
        self.break_group.setTitle(tr['break_settings_group'])
        self.break_method_label.setText(tr['select_break_method'])
        # 更新断点方式下拉框选项
        self.break_method_combo.blockSignals(True)
        self.break_method_combo.clear()
        self.break_method_combo.addItems([tr['manual_input'], tr['natural_breaks']])
        self.break_method_combo.blockSignals(False)
        self.calculate_button.setText(tr['calculate_natural_breaks'])
        self.count_label.setText(tr['select_number_of_classes'])
        self.break_input1_label.setText(tr['variable1_breaks'])
        self.break_input2_label.setText(tr['variable2_breaks'])
        # Tab 2 - 配色矩阵设置
        self.color_group.setTitle(tr['color_matrix_settings_group'])
        self.ramp1_label.setText(tr['variable1_color_ramp'])
        self.ramp2_label.setText(tr['variable2_color_ramp'])
        self.custom_ramp1_input.setPlaceholderText(tr['enter_custom_colors_var1'])
        self.custom_ramp2_input.setPlaceholderText(tr['enter_custom_colors_var2'])
        self.preview_button.setText(tr['preview_color_matrix'])
        self.color_matrix_input_label.setText(tr['color_matrix_input_label'])
        self.color_matrix_preview_label.setText(tr['color_matrix_preview_label'])
        # Tab 2 - 交互式提示标签设置
        self.tooltip_group.setTitle(tr['tooltip_settings_group'])
        self.tooltip_instructions.setText(tr['tooltip_instructions'])
        for i, (field_label, label_label) in enumerate(zip(self.tooltip_field_labels, self.tooltip_label_labels),
                                                       start=1):
            field_label.setText(tr['field_label'].format(num=i))
            label_label.setText(tr['label_label'])
        # Tab 3
        self.generate_button.setText(tr['generate_map'])
        self.save_button.setText(tr['save_map'])
        # 如果已加载 GeoJSON，则更新提示下拉框内容
        if self.gdf is not None:
            cols = list(self.gdf.columns)
            for combo, _ in self.tooltip_selectors:
                current = combo.currentText()
                combo.clear()
                combo.addItems(cols)
                if current in cols:
                    index = cols.index(current)
                    combo.setCurrentIndex(index)

    def load_file(self):
        file_path, _ = QFileDialog.getOpenFileName(self,
                                                   self.translations[self.current_language]['choose_geojson_file'], "",
                                                   "GeoJSON Files (*.geojson)")
        if file_path:
            self.file_path = file_path
            self.file_label.setText(f"{file_path}")
            try:
                self.gdf = gpd.read_file(file_path)
                if self.gdf.crs and self.gdf.crs.to_epsg() != 4326:
                    self.gdf = self.gdf.to_crs(epsg=4326)
            except Exception as e:
                QMessageBox.critical(self, self.translations[self.current_language]['error_title'],
                                     f"{self.translations[self.current_language]['file_read_error']}: {e}")
                self.gdf = None
                return

            all_cols = list(self.gdf.columns)
            if len(all_cols) < 2:
                QMessageBox.warning(self, self.translations[self.current_language]['warning_title'],
                                    self.translations[self.current_language]['insufficient_columns'])
                return

            self.var1_combo.clear()
            self.var2_combo.clear()
            self.var1_combo.addItems(all_cols)
            self.var2_combo.addItems(all_cols)

            for combo, _ in self.tooltip_selectors:
                combo.clear()
                combo.addItems(all_cols)

    def toggle_break_method(self):
        if self.break_method_combo.currentText() == self.translations[self.current_language]['natural_breaks']:
            self.break_input_1.setEnabled(False)
            self.break_input_2.setEnabled(False)
            self.calculate_button.setEnabled(True)
        else:
            self.break_input_1.setEnabled(True)
            self.break_input_2.setEnabled(True)
            self.calculate_button.setEnabled(False)

    def calculate_breaks(self):
        var1 = self.var1_combo.currentText()
        var2 = self.var2_combo.currentText()
        n_classes = self.break_count_spin.value()
        if var1 and var2:
            try:
                breaks1 = jenkspy.jenks_breaks(self.gdf[var1].dropna().values, n_classes=n_classes)
                breaks2 = jenkspy.jenks_breaks(self.gdf[var2].dropna().values, n_classes=n_classes)
                self.break_input_1.setText(",".join(map(str, breaks1)))
                self.break_input_2.setText(",".join(map(str, breaks2)))
                self.breaks1 = breaks1
                self.breaks2 = breaks2
                self.generate_color_matrix()
                QMessageBox.information(self, self.translations[self.current_language]['info_title'],
                                        self.translations[self.current_language]['breaks_calculated'])
            except Exception as e:
                QMessageBox.critical(self, self.translations[self.current_language]['error_title'],
                                     self.translations[self.current_language]['break_calculation_failed'].format(
                                         error=e))

    def toggle_custom_input(self, text, var_index):
        # 如果选择“自定义”（中文）或“Custom”（英文）则显示自定义输入框
        if var_index == 1:
            self.custom_ramp1_input.setVisible(text in ["自定义", "Custom"])
        elif var_index == 2:
            self.custom_ramp2_input.setVisible(text in ["自定义", "Custom"])

    def update_color_matrix_size(self):
        self.color_matrix_input.clear()
        self.update_color_preview()

    def generate_color_matrix(self):
        n_classes = self.break_count_spin.value()
        if self.ramp1_combo.currentText() in ["自定义", "Custom"]:
            custom_colors1 = [c.strip() for c in self.custom_ramp1_input.text().split(",") if c.strip()]
            if len(custom_colors1) != n_classes:
                QMessageBox.warning(self, self.translations[self.current_language]['warning_title'],
                                    f"{self.translations[self.current_language]['variable1_color_ramp']} {n_classes}!")
                return
            base_colors1 = custom_colors1
        else:
            ramp1_name = self.ramp1_combo.currentText()
            base_colors1 = get_expanded_colors(DEFAULT_RAMP_COLORS.get(ramp1_name, []), n_classes)

        if self.ramp2_combo.currentText() in ["自定义", "Custom"]:
            custom_colors2 = [c.strip() for c in self.custom_ramp2_input.text().split(",") if c.strip()]
            if len(custom_colors2) != n_classes:
                QMessageBox.warning(self, self.translations[self.current_language]['warning_title'],
                                    f"{self.translations[self.current_language]['variable2_color_ramp']} {n_classes}!")
                return
            base_colors2 = custom_colors2
        else:
            ramp2_name = self.ramp2_combo.currentText()
            base_colors2 = get_expanded_colors(DEFAULT_RAMP_COLORS.get(ramp2_name, []), n_classes)

        bivariate_colors = []
        for i in range(n_classes):
            row_colors = []
            for j in range(n_classes):
                row_colors.append(blend_colors(base_colors1[i], base_colors2[j]))
            bivariate_colors.append(",".join(row_colors))
        matrix_text = "\n".join(bivariate_colors)
        self.color_matrix_input.setPlainText(matrix_text)
        self.update_color_preview()

    def update_color_preview(self):
        matrix_lines = self.color_matrix_input.toPlainText().splitlines()
        html = "<table border='1' cellpadding='5' cellspacing='0'>"
        for row in matrix_lines:
            colors = row.split(",")
            html += "<tr>"
            for color in colors:
                color = color.strip()
                if color:
                    html += f"<td style='background-color:{color}; width:30px; height:30px;'></td>"
                else:
                    html += "<td></td>"
            html += "</tr>"
        html += "</table>"
        self.color_preview.setText(html)

    def generate_map(self):
        if self.gdf is None or self.gdf.empty:
            QMessageBox.warning(self, self.translations[self.current_language]['warning_title'],
                                self.translations[self.current_language]['please_select_geojson'])
            return

        if not hasattr(self, 'breaks1') or not hasattr(self, 'breaks2'):
            QMessageBox.warning(self, self.translations[self.current_language]['warning_title'],
                                self.translations[self.current_language]['please_calculate_breaks'])
            return

        n_classes = self.break_count_spin.value()
        var1 = self.var1_combo.currentText()
        var2 = self.var2_combo.currentText()

        def classify(value, breaks):
            for i in range(len(breaks) - 1):
                if breaks[i] <= value < breaks[i + 1]:
                    return i
            return len(breaks) - 2

        self.gdf[f"{var1}_class"] = self.gdf[var1].apply(lambda x: classify(x, self.breaks1))
        self.gdf[f"{var2}_class"] = self.gdf[var2].apply(lambda x: classify(x, self.breaks2))
        colors_matrix = [
            row.split(",") for row in self.color_matrix_input.toPlainText().splitlines() if row.strip()
        ]

        def bivariate_color(row):
            index_var1 = int(row[f"{var1}_class"])
            index_var2 = int(row[f"{var2}_class"])
            try:
                return colors_matrix[index_var1][index_var2]
            except IndexError:
                return "#FFFFFF"

        self.gdf["fillColor"] = self.gdf.apply(bivariate_color, axis=1)
        centroid = self.gdf.geometry.centroid
        centroid_y, centroid_x = centroid.y.mean(), centroid.x.mean()

        tooltip_fields = [combo.currentText() for combo, _ in self.tooltip_selectors if combo.currentText()]
        tooltip_aliases = [
            edit.text() if edit.text() else combo.currentText()
            for combo, edit in self.tooltip_selectors
            if combo.currentText()
        ]

        m = folium.Map(location=[centroid_y, centroid_x], zoom_start=5)
        folium.GeoJson(
            data=self.gdf.__geo_interface__,
            style_function=lambda feature: {
                'fillColor': feature["properties"].get("fillColor", "#000000"),
                'color': 'black',
                'weight': 0.5,
                'fillOpacity': 0.7
            },
            tooltip=folium.GeoJsonTooltip(
                fields=tooltip_fields,
                aliases=tooltip_aliases,
                localize=True
            )
        ).add_to(m)

        legend_html = create_bivariate_legend_html(
            breaks1=self.breaks1,
            breaks2=self.breaks2,
            color_matrix=colors_matrix,
            var1_name=var1,
            var2_name=var2,
            legend_title=self.translations[self.current_language]['legend_title']
        )
        m.get_root().html.add_child(folium.Element(legend_html))
        m.save(self.map_html)
        self.map_preview.load(QUrl.fromLocalFile(os.path.abspath(self.map_html)))
        QMessageBox.information(self, self.translations[self.current_language]['info_title'],
                                self.translations[self.current_language]['map_generated_info'])
        self.save_button.setEnabled(True)

    def save_map(self):
        save_path, _ = QFileDialog.getSaveFileName(self, self.translations[self.current_language]['save_map'],
                                                   "bivariate_map.html", "HTML Files (*.html)")
        if save_path:
            try:
                import shutil
                shutil.copy(self.map_html, save_path)
                QMessageBox.information(self, self.translations[self.current_language]['info_title'],
                                        self.translations[self.current_language]['map_saved_info'].format(
                                            path=save_path))
            except Exception as e:
                QMessageBox.critical(self, self.translations[self.current_language]['error_title'],
                                     self.translations[self.current_language]['save_failed'].format(error=e))


if __name__ == "__main__":
    app = QApplication(sys.argv)
    # 在 QApplication 初始化后设置应用程序图标，这将影响任务栏图标
    app.setWindowIcon(QIcon(r"D:\Jupyter\遗产灾害\icon.ico"))
    window = BivariateMapApp()
    window.show()
    sys.exit(app.exec_())